@@ -1,6 +1,10 @@
-use hudsucker::{rustls::internal::pemfile, *};
+use hudsucker::{
+    hyper::{Body, Response},
+    rustls::internal::pemfile,
+    *,
+};
 use log::*;
-use std::net::SocketAddr;
+use std::{future::Future, net::SocketAddr, pin::Pin};
 
 async fn shutdown_signal() {
     tokio::signal::ctrl_c()
@@ -12,8 +16,11 @@ async fn shutdown_signal() {
 async fn main() {
     env_logger::init();
 
-    let request_handler = |req| RequestOrResponse::Request(req);
-    let response_handler = |res| res;
+    let request_handler = |req| -> Pin<Box<dyn Future<Output = RequestOrResponse> + Send>> {
+        Box::pin(async { RequestOrResponse::Request(req) })
+    };
+    let response_handler =
+        |res| -> Pin<Box<dyn Future<Output = Response<Body>> + Send>> { Box::pin(async { res }) };
     let incoming_message_handler = |msg| Some(msg);
     let outgoing_message_handler = |msg| Some(msg);
 
